% adctest           - Fit a sine wave to the iGp data
%
% Copyright (c) 2006 Dimtel, Inc., All Rights Reserved
%
% $Id: adctest.m,v 1.3 2009/02/06 00:10:45 dim Exp $
% $Date: 2009/02/06 00:10:45 $
% $Author: dim $
% $Revision: 1.3 $
% $Log: adctest.m,v $
% Revision 1.3  2009/02/06 00:10:45  dim
% Sinewave fitting improvements, processing downsampling and beam current added
%
% Revision 1.2  2008/01/05 20:48:23  dim
% Bug fixes for better fitting
%
% Revision 1.1  2007/10/28 08:23:15  dim
% Tools to include in the distribution
%
% Revision 1.2  2007/05/14 17:22:04  dim
% Added stop condition on max error
%
% Revision 1.1  2007/03/14 08:35:58  dim
% Matlab tools for data acquisition and testing with file-based data exchange
%
%

function [sol, RMS, D, amp, fr, x, x_fit] = adctest(sys, count, stop_max)

pvroot=['IGPF:', sys, ':'];

Ts = 1/lcaGet([pvroot, 'RF_FREQ'])/1e6;
opts=optimset('TolX', 1e-6, 'TolFun', 1e-6, 'LargeScale', 'off');

for k=1:count

  x = get_data(pvroot);

  N = length(x);
  t = [0:N-1]*Ts;
  [h, freq] = psd(x, 2^16, 1/Ts/1e6);
  h = h/max(h);
  if (k == 1)
    h_max = h;
  else
    h_max = max(h, h_max);
  end

  % Do the fitting
  [sol, x_fit] = sinfit(Ts, x, opts);

  amp(k) = sol(1);
  fr(k) = sol(2);
  RMS(k) = std(x-x_fit, 1);
  [D(k), peak_idx(k)]   = max(abs(x-x_fit));

  figure(1); clf
  subplot(211)
  plot(t*1e6, x, t*1e6, x_fit, 'r');
  xlabel('Time ({\mu}s)');
  ylabel('Amplitude (counts)');
  subplot(212);
%  hist(x-x_fit, 50);
  plot(x_fit, x-x_fit, '.');

  figure(2); clf
  plot(freq, 10*log10(h_max), 'r', freq, 10*log10(h), 'b');
  xlabel('Frequency (MHz)');
  ylabel('Amplitude (dB)');
  grid;

  figure(3); clf
  subplot(221);
  plot(1:k, RMS(1:k), 'ro-');
  xlabel('Acquisition');
  ylabel('RMS error (ADC counts)');
  grid;
  subplot(223);
  plot(1:k, D(1:k), '*-');
  xlabel('Acquisition');
  ylabel('Maximum error (ADC counts)');
  grid;
  subplot(222);
  plot(1:k, amp(1:k), 'ro-');
  xlabel('Acquisition');
  ylabel('Amplitude (ADC counts)');
  grid;
  subplot(224);
  plot(1:k, fr(1:k)/1e3, '*-');
  xlabel('Acquisition');
  ylabel('Frequency (kHz)');
  grid;

  drawnow;
%  if (k ~= count)
%    pause(1);
%  end
  if (nargin == 3)
    if (D(k) > stop_max)
      break;
    end
  end
end
